/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef REGISTERER_HPP
#define REGISTERER_HPP

#include "registry_base.hpp"

namespace curlpp
{
   /**
    * This class is used to register a handle to many registries. This is
    * mostly used to permit to have a base configuration for you requests. A
    * good example of it's use is if you're behind a proxy. On libcurl you are
    * obliged to set all the proxy options individualy to each handle, but
    * with this you don't need to. You register your handle to the registerer
    * and he will register your ahandle to all the registries you added to the
    * registerer.
    */
   template< typename T >
   class registerer 
   {
      public:
         /**
          * This function add a registry to the registerer. Each handles that
          * will be registered to the registerer will be registered to the
          * registry passed in argument. 
          *
          * NOTE: if you add a registry after you recorded many handles,
          * theses handles will be recorded to this registry. 
          */
         virtual void record_registry( curlpp::registry_base< T > * registry);

         /**
          * This function remove a registry, previously added to the
          * registerer. Each handles that was registered to the registry by
          * the registerer (and the registerer only) will be unregistered if
          * you set unregister_handles to true.
          */
         virtual void unrecord_registry( registry_base< T > *handle, bool
               unregister_handles = false );

         /**
          * register to all registries that were previously added by the
          * function curlpp::registerer::add( registry_base< T > * ).
          */
         virtual void record( const T &handle);
      
         /** 
          * unregister the handle to all registries that he was registered
          * before by the curlpp::registerer::record( T * ) call.
          */
         virtual void unrecord( const T &handle);
      
         virtual ~registerer();

      private:
         std::list< curlpp::registry_base< T > * > m_registry_list;
         std::list< T > m_handles_list;
   };
}

#include "registerer.inl"

#endif
