/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "registry_base.hpp"
#include "utilspp/type_trait.hpp"

namespace curlpp
{

   template< typename U >
      struct pointer_on_member_function
      {
         enum { result = true };
         typedef U type;
         typedef ::utilspp::null_type param_type;
         typedef ::utilspp::null_type class_type;
         typedef ::utilspp::null_type result_type;
      };

   template< typename V, typename W, typename R >
      struct pointer_on_member_function< R(V::*)( W ) >
      {
         enum { result = true };
         //typedef R(V::*)( W ) type;
         typedef W param_type;
         typedef V class_type;
         typedef R result_type;
      };

   /**
    * This class is the class who really updates registered clients.
    * But see curlpp::registry_base< T > for more documentation.
    */
   template
      <
         typename T,
         T function
      >
      class registry : public registry_base
      < 
         typename pointer_on_member_function< T >::class_type
      >
      {
         private:

         public:
            registry();

            /**
             * This is by this function that you change the value for the registry.
             * a copy of the value passed will be kept.
             */
            void set
               ( typename curlpp::pointer_on_member_function< T >::param_type param );

         protected:
            /**
             * This is this function who really refresh the handle.See
             * curlpp::registry_base< T >::refresh( T  *) for more documentation.
             */
            virtual void refresh
               ( typename pointer_on_member_function< T >::class_type *handle );

         private:
            T m_function;
            typename ::utilspp::type_trait
               < 
                  typename ::utilspp::type_trait
                  < 
                     typename curlpp::pointer_on_member_function< T >::param_type
                  >::non_reference 
               >::non_const m_param;
      };

   /**
    * This class is the class who really updates registered clients.
    * But see curlpp::registry_base< T > for more documentation.
    */
   template
      <
         typename T,
         T function
      >
      class registry_free : public registry_base
      < 
         typename pointer_on_member_function< T >::class_type 
      >
      {
         public:
            registry_free();

            /**
             * This is by this function that you change the value for the registry.
             */
            void set();

         protected:
            /**
             * This is this function who really refresh the handle.See
             * curlpp::registry_base< T >::refresh( T  *) for more documentation.
             */
            virtual void refresh( typename pointer_on_member_function< T >::class_type *handle );

         private:
            T m_function;
      };

};

#include "registry.inl"

