/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (utilspp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef MUTEX_FACTORY_HPP
#define MUTEX_FACTORY_HPP

#include "../singleton.hpp"

namespace utilspp
{
   class factory_mutex_base;
   
   class mutex_factory_base
   {
      public:
         virtual ~mutex_factory_base();
         virtual utilspp::factory_mutex_base *create_mutex() = 0;
   };
        
   template< class T >
   class mutex_factory_impl : public mutex_factory_base
   {
      public:
         virtual utilspp::factory_mutex_base *create_mutex();
   };
   
   class mutex_factory_container : public mutex_factory_base
   {
      public:
         mutex_factory_container();
         ~mutex_factory_container();
         
         template< class T >
         void initialize();

         virtual utilspp::factory_mutex_base *create_mutex();

      private:
         mutex_factory_base *m_factory;
   };

   typedef singleton_holder
      <
         utilspp::mutex_factory_container,
         utilspp::creation_using_new,
         utilspp::lifetime_default,
         utilspp::threading_single
      > mutex_factory;
};

#endif
