/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef LIFETIME_LIBRARY_HPP
#define LIFETIME_LIBRARY_HPP

#include <cassert>

#include "singleton_holder.hpp"

namespace utilspp
{
   
   template< typename T >
   unsigned int get_longevity( T *p );

   /**
    * Assigns an object a longevity. Ensures ordered destructions of objects
    * registered thusly during the exit sequence of the application.
    */
   template< typename T, typename T_destroyer >
      void set_library_longevity( 
            T *obj, 
            unsigned int longevity, 
            T_destroyer d = utilspp::private_members::deleter< T >::delete_object
            );

   template< typename T >
      struct lifetime_library
      {
         static void schedule_destruction( T *obj, void (*func)() );
         static void on_dead_reference();
      };

      class lifetime_library_impl
      {
         public:
            lifetime_library_impl();
            ~lifetime_library_impl();
            
            void add( utilspp::private_members::lifetime_tracker *tracker );
            void terminate();

         private:
            utilspp::private_members::tracker_array m_tracker_array;
            int m_nb_elements;
      };

      typedef utilspp::singleton_holder< 
         utilspp::lifetime_library_impl,
         utilspp::creation_using_new,  
         utilspp::lifetime_with_longevity
         > lifetime_library_singleton;

      unsigned int get_longevity( utilspp::lifetime_library_impl *p );
};

#include "lifetime_library.inl"

#endif
