/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef SMARTPTR_HPP
#define SMARTPTR_HPP

#include "../exception.hpp"

#define NULL_BODY_ERROR "the smart pointer contain a NULL pointer"

namespace utilspp
{
   class non_copyable
   {
      public:
         non_copyable()
         {}

      private:
         non_copyable( const non_copyable& r )
         {}
   };

   template <class type = unsigned int>
      class fast_count
      {
         public:
            typedef type counted_type;

            fast_count( type count = 1 ) : _count( count )
            {}

            fast_count& operator++()
            {
               _count++;
               return ( *this );
            }

            fast_count& operator--()
            {
               _count--;
               return ( *this );
            }

            operator type()
            {
               return _count;
            }

            counted_type use_count()
            {
               return _count;
            }

         private:
            type _count;
      };

   template < class content_type /*, typename count_type = unsigned int,*/ /*template < class > class count_policy = fast_count*/ >
      class counting_body : public utilspp::non_copyable
      {
         public:
            counting_body( content_type *body ) : _body( body )
            {}

            void inc()
            {
               ++_count;
            }

            void dec()
            {
               --_count;
               if ( _count <= 0 )
               {
                  if ( _body != NULL )
                  {
                     delete _body;
                     _body = NULL;
                  }
                  delete this;
               }
            }

            /*
               content_type& operator*()
               {
               runtime_assert(_body,NULL_BODY_ERROR);
               return (*_body);
               }

               content_type* operator->()
               {
               runtime_assert(_body,NULL_BODY_ERROR);
               return _body;
               }

             */
            content_type* get
               ()
               {
                  return _body;
               }

         protected:
            ~counting_body()
            {}

         private:
            utilspp::fast_count<unsigned int> _count;
            content_type* _body;
      };


   template < class content_type >
      class shared_ptr
      {
         public:
            shared_ptr();

            explicit shared_ptr( content_type *content ) : _content( new
                  utilspp::counting_body< content_type >( content ) )
            {}

            ~shared_ptr()
            {
               _content->dec();
            }

            shared_ptr( const shared_ptr& other ) : _content( other._content )
            {
               _content->inc();
            }

            shared_ptr& operator=( const shared_ptr& other )
            {
               if ( _content->get
                     () != other._content->get
                     () )
               {
                  _content->dec();
                  _content = other._content;
                  _content->inc();
               }
               return ( *this );
            }

            //me semble que c<es tpas safe ca car ilo pourrait utiliser "operator const content_type*()"
            //pis tout fourrer
            /*
               shared_ptr& operator=(content_type *content)
               {
               _content--;
               _content = new counting_body< content_type >(content);
               }*/

            bool operator==( const shared_ptr& other ) const
            {
               return ( _content->get
                     () == other._content->get
                     () );
            }

            bool operator!=( const shared_ptr& other ) const
            {
               return ( _content->get
                     () != other._content->get
                     () );
            }

            bool operator<( const shared_ptr& other ) const
            {
               return ( _content->get() < other._content->get() );
            }

            operator content_type*()
            {
               return _content->get
                  ();
            }

            content_type& operator*()
            {
               runtime_assert( _content->get
                     (), NULL_BODY_ERROR );
               return ( *_content->get
                     () );
            }

            content_type* operator->()
            {
               runtime_assert( _content->get
                     (), NULL_BODY_ERROR );
               return _content->get
                  ();
            }

            /*
             * content_type* get
             ()
             {
             return _content->get
             ();
             }
             */

         private:
            utilspp::counting_body< content_type > * _content;
      };
};

template < class content_type >
utilspp::shared_ptr< content_type >::shared_ptr() : _content(
      new utilspp::counting_body< content_type >( NULL ) )
{}

#endif
