/*
*    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
*    
*    Permission is hereby granted, free of charge, to any person obtaining
*    a copy of this software and associated documentation files 
*    (curlpp), to deal in the Software without restriction, 
*    including without limitation the rights to use, copy, modify, merge,
*    publish, distribute, sublicense, and/or sell copies of the Software,
*    and to permit persons to whom the Software is furnished to do so, 
*    subject to the following conditions:
*    
*    The above copyright notice and this permission notice shall be included
*    in all copies or substantial portions of the Software.
*    
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "curlpp.hpp"
#include <fstream>

class my_progress_trait : public curlpp::progress_trait
{
   public:
      bool progress_function(
         double dltotal,
         double dlnow,
         double ultotal,
         double ulnow
      );
};

bool my_progress_trait::progress_function(
   double dltotal,
   double dlnow,
   double ultotal,
   double ulnow
)
{
   if ( dltotal != 0 )
   {
      std::cout << "Downloaded " << dlnow / dltotal * 100 << "\% on " << dltotal / 1024 << " KBytes" << std::endl;
   }
   else
   {
      std::cout << "Uploaded " << ulnow / ultotal*100 << "\% on " << ultotal / 1024 << " KBytes" << std::endl;
   }
   
   return true;
}

int main()
{
   try
   {
      curlpp::cleanup cleaner;
      curlpp::easy request;
      request.url( "ftp://ftp.kernel.org/pub/linux/kernel/v2.4/linux-2.4.19.tar.gz" );
      
      std::ofstream file( "linux-2.4.29.tar.gz" );
      curlpp::ostream_trait body_trait( &file );
      request.m_body_storage.trait( &body_trait );
      
      request.progress();
      my_progress_trait progress_trait;
      request.m_progress_storage.trait( &progress_trait );
      
      request.perform();
   }
   catch ( curlpp::exception & e )
   {
      std::cout << e.what() << std::endl;
   }
   return 0;
}
